@@ -7,6 +7,7 @@
   (:use #:cl)
   (:export #:*version-string*
            #:*dry-run*
+           #:*on-error-stop*
            #:*self-upgrade-immutable-systems*
 	   #:*fd-path-root*
 	   #:*root-dir*
@@ -15,34 +16,40 @@
 	   #:*client-min-messages*
 	   #:*log-min-messages*
            #:*report-stream*
+           #:*pgsql-reserved-keywords*
            #:*identifier-case*
            #:*preserve-index-names*
 	   #:*copy-batch-rows*
            #:*copy-batch-size*
            #:*concurrent-batches*
 	   #:*pg-settings*
-	   #:*state*
 	   #:*default-tmpdir*
 	   #:init-params-from-environment
-	   #:getenv-default))
+	   #:getenv-default
+
+           #:+os-code-success+
+           #:+os-code-error+
+           #:+os-code-error-usage+
+           #:+os-code-error-bad-source+
+           #:+os-code-error-regress+))
 
 (in-package :pgloader.params)
 
 (defparameter *release* t
   "non-nil when this build is a release build.")
 
-(defparameter *major-version* "3.2")
+(defparameter *major-version* "3.3")
 (defparameter *minor-version* "2")
 
 (defun git-hash ()
   "Return the current abbreviated git hash of the development tree."
   (handler-case
       (let ((git-hash `("git" "--no-pager" "log" "-n1" "--format=format:%h")))
-        (uiop:with-current-directory ((asdf:system-source-directory :pgloader))
-          (multiple-value-bind (stdout stderr code)
-              (uiop:run-program git-hash :output :string)
-            (declare (ignore code stderr))
-            stdout)))
+        (multiple-value-bind (stdout stderr code)
+            (uiop:run-program git-hash :output :string
+                              :directory (asdf:system-source-directory :pgloader))
+          (declare (ignore code stderr))
+          stdout))
     (condition (e)
       ;; in case anything happen, just return X.Y.Z~devel
       (declare (ignore e))
@@ -68,18 +75,16 @@
 (defparameter *dry-run* nil
   "Set to non-nil to only run checks about the load setup.")
 
-;; we can't use pgloader.utils:make-pgstate yet because params is compiled
-;; first in the asd definition, we just make the symbol a special variable.
-(defparameter *state* nil
-  "State of the current loading.")
+(defparameter *on-error-stop* nil
+  "Set to non-nil to for pgloader to refrain from handling errors, quitting instead.")
 
 (defparameter *fd-path-root* nil
   "Where to load files from, when loading from an archive or expanding regexps.")
 
 (defparameter *root-dir*
-  #+unix (make-pathname :directory "/tmp/pgloader/")
+  #+unix (uiop:parse-native-namestring "/tmp/pgloader/")
   #-unix (uiop:merge-pathnames*
-          "pgloader/"
+          (uiop:make-pathname* :direction '(:relative "pgloader"))
           (uiop:ensure-directory-pathname (getenv-default "Temp")))
   "Top directory where to store all data logs and reject files.")
 
@@ -100,6 +105,9 @@
 ;;;
 ;;; When converting from other databases, how to deal with case sensitivity?
 ;;;
+(defvar *pgsql-reserved-keywords* nil
+  "We need to always quote PostgreSQL reserved keywords")
+
 (defparameter *identifier-case* :downcase
   "Dealing with source databases casing rules.")
 
@@ -144,3 +152,13 @@
   (setf *default-tmpdir*
 	(fad:pathname-as-directory
 	 (getenv-default "TMPDIR" *default-tmpdir*))))
+
+;;;
+;;; Some command line constants for OS errors codes
+;;;
+(defparameter +os-code-success+          0)
+(defparameter +os-code-error+            1)
+(defparameter +os-code-error-usage+      2)
+(defparameter +os-code-error-bad-source+ 4)
+(defparameter +os-code-error-regress+    5)
+
