@@ -54,11 +54,14 @@
   (def-keyword-rule "rows")
   (def-keyword-rule "size")
   (def-keyword-rule "concurrency")
+  (def-keyword-rule "max")
+  (def-keyword-rule "parallel")
   (def-keyword-rule "reject")
   (def-keyword-rule "file")
   (def-keyword-rule "log")
   (def-keyword-rule "level")
   (def-keyword-rule "encoding")
+  (def-keyword-rule "timezone")
   (def-keyword-rule "decoding")
   (def-keyword-rule "truncate")
   (def-keyword-rule "disable")
@@ -99,7 +102,9 @@
   (def-keyword-rule "schemas")
   (def-keyword-rule "only")
   (def-keyword-rule "drop")
+  (def-keyword-rule "alter")
   (def-keyword-rule "create")
+  (def-keyword-rule "rename")
   (def-keyword-rule "materialize")
   (def-keyword-rule "reset")
   (def-keyword-rule "table")
