@@ -14,9 +14,12 @@
 ;;;
 (make-option-rule create-schemas (and kw-create (? kw-no) kw-schemas))
 
-(defrule mssql-option (or option-batch-rows
+(defrule mssql-option (or option-workers
+                          option-concurrency
+                          option-batch-rows
                           option-batch-size
                           option-batch-concurrency
+                          option-max-parallel-create-index
                           option-truncate
                           option-disable-triggers
                           option-data-only
@@ -26,35 +29,13 @@
                           option-create-schemas
                           option-create-indexes
                           option-reset-sequences
+			  option-foreign-keys
                           option-encoding
                           option-identifiers-case))
 
-(defrule another-mssql-option (and comma mssql-option)
-  (:lambda (source)
-    (bind (((_ option) source)) option)))
-
-(defrule mssql-option-list (and mssql-option (* another-mssql-option))
-  (:lambda (source)
-    (destructuring-bind (opt1 opts) source
-      (alexandria:alist-plist (list* opt1 opts)))))
-
-(defrule mssql-options (and kw-with mssql-option-list)
-  (:lambda (source)
-    (bind (((_ opts) source))
-      (cons :mssql-options opts))))
-
-(defrule like-expression (and "'" (+ (not "'")) "'")
-  (:lambda (le)
-    (bind (((_ like _) le)) (text like))))
-
-(defrule another-like-expression (and comma like-expression)
-  (:lambda (source)
-    (bind (((_ like) source)) like)))
-
-(defrule filter-list-like (and like-expression (* another-like-expression))
-  (:lambda (source)
-    (destructuring-bind (filter1 filters) source
-      (list* filter1 filters))))
+(defrule mssql-options (and kw-with
+                            (and mssql-option (* (and comma mssql-option))))
+  (:function flatten-option-list))
 
 (defrule including-in-schema
     (and kw-including kw-only kw-table kw-names kw-like filter-list-like
@@ -63,7 +44,8 @@
     (bind (((_ _ _ _ _ filter-list _ _ schema) source))
       (cons schema filter-list))))
 
-(defrule including-like (and including-in-schema (* including-in-schema))
+(defrule including-like-in-schema
+    (and including-in-schema (* including-in-schema))
   (:lambda (source)
     (destructuring-bind (inc1 incs) source
       (cons :including (list* inc1 incs)))))
@@ -75,7 +57,8 @@
     (bind (((_ _ _ _ filter-list _ _ schema) source))
       (cons schema filter-list))))
 
-(defrule excluding-like (and excluding-in-schema (* excluding-in-schema))
+(defrule excluding-like-in-schema
+    (and excluding-in-schema (* excluding-in-schema))
   (:lambda (source)
     (destructuring-bind (excl1 excls) source
       (cons :excluding (list* excl1 excls)))))
@@ -87,10 +70,12 @@
 (defrule load-mssql-optional-clauses (* (or mssql-options
                                             gucs
                                             casts
+                                            alter-schema
+                                            alter-table
                                             before-load
                                             after-load
-                                            including-like
-                                            excluding-like))
+                                            including-like-in-schema
+                                            excluding-like-in-schema))
   (:lambda (clauses-list)
     (alexandria:alist-plist clauses-list)))
 
@@ -142,26 +127,25 @@
 ;;; LOAD DATABASE FROM mssql://
 (defun lisp-code-for-mssql-dry-run (ms-db-conn pg-db-conn)
   `(lambda ()
+     ;; now is the time to load the CFFI lib we need (freetds)
+     (log-message :log "Loading the FreeTDS shared librairy (sybdb)")
+     (cffi:load-foreign-library 'mssql::sybdb)
+
      (log-message :log "DRY RUN, only checking connections.")
      (check-connection ,ms-db-conn)
      (check-connection ,pg-db-conn)))
 
 (defun lisp-code-for-loading-from-mssql (ms-db-conn pg-db-conn
                                          &key
-                                           gucs casts before after
-                                           ((:mssql-options options))
-                                           (including)
-                                           (excluding))
+                                           gucs casts before after options
+                                           alter-schema alter-table
+                                           including excluding)
   `(lambda ()
      ;; now is the time to load the CFFI lib we need (freetds)
      (let (#+sbcl(sb-ext:*muffled-warnings* 'style-warning))
        (cffi:load-foreign-library 'mssql::sybdb))
 
-     (let* ((state-before  (pgloader.utils:make-pgstate))
-            (*state*       (or *state* (pgloader.utils:make-pgstate)))
-            (state-idx     (pgloader.utils:make-pgstate))
-            (state-after   (pgloader.utils:make-pgstate))
-            (*default-cast-rules* ',*mssql-default-cast-rules*)
+     (let* ((*default-cast-rules* ',*mssql-default-cast-rules*)
             (*cast-rules*         ',casts)
             ,@(pgsql-connection-bindings pg-db-conn gucs)
             ,@(batch-control-bindings options)
@@ -171,29 +155,24 @@
                             :target-db ,pg-db-conn
                             :source-db ,ms-db-conn)))
 
-       ,(sql-code-block pg-db-conn 'state-before before "before load")
+       ,(sql-code-block pg-db-conn :pre before "before load")
 
        (pgloader.mssql:copy-database source
-                                     :state-before state-before
-                                     :state-after state-after
-                                     :state-indexes state-idx
                                      :including ',including
                                      :excluding ',excluding
+                                     :alter-schema ',alter-schema
+                                     :alter-table ',alter-table
+                                     :set-table-oids t
                                      ,@(remove-batch-control-option options))
 
-       ,(sql-code-block pg-db-conn 'state-after after "after load")
-
-       (report-full-summary "Total import time" *state*
-                            :before   state-before
-                            :finally  state-after
-                            :parallel state-idx))))
+       ,(sql-code-block pg-db-conn :post after "after load"))))
 
 (defrule load-mssql-database load-mssql-command
   (:lambda (source)
     (bind (((ms-db-uri pg-db-uri
                        &key
-                       gucs casts before after including excluding
-                       ((:mssql-options options)))
+                       gucs casts before after alter-schema alter-table
+                       including excluding options)
             source))
       (cond (*dry-run*
              (lisp-code-for-mssql-dry-run ms-db-uri pg-db-uri))
@@ -203,7 +182,9 @@
                                                :casts casts
                                                :before before
                                                :after after
-                                               :mssql-options options
+                                               :alter-schema alter-schema
+                                               :alter-table alter-table
+                                               :options options
                                                :including including
                                                :excluding excluding))))))
 
