@@ -13,18 +13,12 @@ load database
 
   set work_mem to '16MB', maintenance_work_mem to '512 MB';
 |#
-(defrule option-encoding (and kw-encoding encoding)
-  (:lambda (enc)
-    (cons :encoding
-          (if enc
-              (destructuring-bind (kw-encoding encoding) enc
-                (declare (ignore kw-encoding))
-                encoding)
-              :utf-8))))
-
-(defrule sqlite-option (or option-batch-rows
+(defrule sqlite-option (or option-workers
+                           option-concurrency
+                           option-batch-rows
                            option-batch-size
                            option-batch-concurrency
+                           option-max-parallel-create-index
                            option-truncate
                            option-disable-triggers
 			   option-data-only
@@ -33,21 +27,24 @@ load database
 			   option-create-tables
 			   option-create-indexes
 			   option-reset-sequences
+                           option-foreign-keys
                            option-encoding))
 
-(defrule another-sqlite-option (and comma sqlite-option)
-  (:lambda (source)
-    (bind (((_ option) source)) option)))
+(defrule sqlite-options (and kw-with
+                             (and sqlite-option (* (and comma sqlite-option))))
+  (:function flatten-option-list))
 
-(defrule sqlite-option-list (and sqlite-option (* another-sqlite-option))
+(defrule including-like
+    (and kw-including kw-only kw-table kw-names kw-like filter-list-like)
   (:lambda (source)
-    (destructuring-bind (opt1 opts) source
-      (alexandria:alist-plist (list* opt1 opts)))))
+    (bind (((_ _ _ _ _ filter-list) source))
+      (cons :including filter-list))))
 
-(defrule sqlite-options (and kw-with sqlite-option-list)
+(defrule excluding-like
+    (and kw-excluding kw-table kw-names kw-like filter-list-like)
   (:lambda (source)
-    (bind (((_ opts) source))
-      (cons :sqlite-options opts))))
+    (bind (((_ _ _ _ filter-list) source))
+      (cons :excluding filter-list))))
 
 (defrule sqlite-db-uri (and "sqlite://" filename)
   (:lambda (source)
@@ -77,8 +74,10 @@ load database
 (defrule load-sqlite-optional-clauses (* (or sqlite-options
                                              gucs
                                              casts
-                                             including-matching
-                                             excluding-matching))
+                                             including-like
+                                             excluding-like
+                                             before-load
+                                             after-load))
   (:lambda (clauses-list)
     (alexandria:alist-plist clauses-list)))
 
@@ -96,34 +95,37 @@ load database
 
 (defun lisp-code-for-loading-from-sqlite (sqlite-db-conn pg-db-conn
                                           &key
-                                            gucs casts
-                                            ((:sqlite-options options))
+                                            gucs casts before after options
                                             ((:including incl))
                                             ((:excluding excl)))
   `(lambda ()
-     (let* ((state-before   (pgloader.utils:make-pgstate))
-            (*state*        (pgloader.utils:make-pgstate))
-            (*default-cast-rules* ',*sqlite-default-cast-rules*)
+     (let* ((*default-cast-rules* ',*sqlite-default-cast-rules*)
             (*cast-rules*         ',casts)
             ,@(pgsql-connection-bindings pg-db-conn gucs)
             ,@(batch-control-bindings options)
-            (source-db      (with-stats-collection ("fetch" :state state-before)
+            (source-db      (with-stats-collection ("fetch" :section :pre)
                                 (expand (fetch-file ,sqlite-db-conn))))
             (source
              (make-instance 'pgloader.sqlite::copy-sqlite
                             :target-db ,pg-db-conn
                             :source-db source-db)))
+
+       ,(sql-code-block pg-db-conn :pre before "before load")
+
        (pgloader.sqlite:copy-database source
-                                      :state-before state-before
+                                      :set-table-oids t
                                       :including ',incl
                                       :excluding ',excl
-                                      ,@(remove-batch-control-option options)))))
+                                      ,@(remove-batch-control-option options))
+
+       ,(sql-code-block pg-db-conn :post after "after load"))))
 
 (defrule load-sqlite-database load-sqlite-command
   (:lambda (source)
     (destructuring-bind (sqlite-uri
                          pg-db-uri
-                         &key gucs casts sqlite-options including excluding)
+                         &key
+                         gucs casts before after options including excluding)
         source
       (cond (*dry-run*
              (lisp-code-for-sqlite-dry-run sqlite-uri pg-db-uri))
@@ -131,7 +133,9 @@ load database
              (lisp-code-for-loading-from-sqlite sqlite-uri pg-db-uri
                                                 :gucs gucs
                                                 :casts casts
-                                                :sqlite-options sqlite-options
+                                                :before before
+                                                :after after
+                                                :options options
                                                 :including including
                                                 :excluding excluding))))))
 
