@@ -9,8 +9,12 @@
     (:source (:type "nchar")     :target (:type "text" :drop-typemod t))
     (:source (:type "varchar")   :target (:type "text" :drop-typemod t))
     (:source (:type "nvarchar")  :target (:type "text" :drop-typemod t))
+    (:source (:type "ntext")     :target (:type "text" :drop-typemod t))
     (:source (:type "xml")       :target (:type "text" :drop-typemod t))
 
+    (:source (:type "int" :auto-increment t)
+             :target (:type "bigserial" :drop-default t))
+
     (:source (:type "bit") :target (:type "boolean")
              :using pgloader.transforms::sql-server-bit-to-boolean)
 
@@ -49,6 +53,9 @@
     (:source (:type "binary") :target (:type "bytea")
              :using pgloader.transforms::byte-vector-to-bytea)
 
+    (:source (:type "image") :target (:type "bytea")
+             :using pgloader.transforms::byte-vector-to-bytea)
+
     (:source (:type "varbinary") :target (:type "bytea")
              :using pgloader.transforms::byte-vector-to-bytea)
 
@@ -80,11 +87,7 @@
 (defmethod mssql-column-ctype ((col mssql-column))
   "Build the ctype definition from the full mssql-column information."
   (let ((type (mssql-column-type col)))
-    (cond ((and (string= type "int")
-                (mssql-column-identity col))
-           "bigserial")
-
-          ((member type '("float" "real") :test #'string=)
+    (cond ((member type '("float" "real") :test #'string=)
            ;; see https://msdn.microsoft.com/en-us/library/ms173773.aspx
            ;; scale is supposed to be nil, and useless in PostgreSQL, so we
            ;; just ignore it
@@ -102,27 +105,39 @@
 
           (t type))))
 
-(defmethod format-pgsql-column ((col mssql-column))
-  "Return a string representing the PostgreSQL column definition."
-  (let* ((column-name (apply-identifier-case (mssql-column-name col)))
-	 (type-definition
-	  (with-slots (schema table-name name type default nullable)
-	      col
-            (declare (ignore schema))   ; FIXME
-            (let ((ctype (mssql-column-ctype col)))
-              (cast table-name name type ctype default nullable nil)))))
-    (format nil "~a ~22t ~a" column-name type-definition)))
+(defmethod cast ((field mssql-column))
+  "Return the PostgreSQL type definition from given MS SQL column definition."
+  (with-slots (schema table-name name type default nullable)
+      field
+    (declare (ignore schema))   ; FIXME
+    (let* ((ctype (mssql-column-ctype field))
+           (extra (when (mssql-column-identity field) "auto_increment"))
+           (pgcol
+            (apply-casting-rules table-name name type ctype default nullable extra)))
+      ;; the MS SQL driver smartly maps data to the proper CL type, but the
+      ;; pgloader API only wants to see text representations to send down the
+      ;; COPY protocol.
+      (unless (column-transform pgcol)
+        (setf (column-transform pgcol)
+              (lambda (val) (if val (format nil "~a" val) :null))))
 
-(defun cast-mssql-column-definition-to-pgsql (mssql-column)
-  "Return the PostgreSQL column definition from the MS SQL one."
-  (multiple-value-bind (column fn)
-      (with-slots (schema table-name name type default nullable)
-          mssql-column
-        (declare (ignore schema))       ; FIXME
-        (let ((ctype (mssql-column-ctype mssql-column)))
-          (cast table-name name type ctype default nullable nil)))
+      ;; normalize default values
+      ;; see *pgsql-default-values*
+      (setf (column-default pgcol)
+            (cond ((null default) :null)
+                  ((and (stringp default) (string= "NULL" default)) :null)
+
+                  ((and (stringp default)
+                        ;; address CURRENT_TIMESTAMP(6) and other spellings
+                        (or (uiop:string-prefix-p "CURRENT_TIMESTAMP" default)
+                            (string= "CURRENT TIMESTAMP" default)))
+                   :current-timestamp)
+
+                  ((and (stringp default)
+                        (or (string= "newid()" default)
+                            (string= "newsequentialid()" default)))
+                   :generate-uuid)
+
+                  (t default)))
+      pgcol)))
 
-    ;; the MS SQL driver smartly maps data to the proper CL type, but the
-    ;; pgloader API only wants to see text representations to send down the
-    ;; COPY protocol.
-    (values column (or fn (lambda (val) (if val (format nil "~a" val) :null))))))
